@@ -2,6 +2,7 @@ import struct, socket, threading, json, os, pickle
 from essentials import tokening
 import essentials
 import copy
+import time
 
 def SocketDownload(sock):
     """
@@ -10,18 +11,17 @@ def SocketDownload(sock):
     try:
         data = b""
         payload_size = struct.calcsize(">L")
-        while True:
-            while len(data) < payload_size:
-                data += sock.recv(4096)
-            packed_msg_size = data[:payload_size]
-            data = data[payload_size:]
-            msg_size = struct.unpack(">L", packed_msg_size)[0]
-            while len(data) < msg_size:
-                data += sock.recv(4096)
-            frame_data = data[:msg_size]
-            data = data[msg_size:]
-            xData = pickle.loads(frame_data, fix_imports=True, encoding="bytes")
-            return xData
+        while len(data) < payload_size:
+            data += sock.recv(4096)
+        packed_msg_size = data[:payload_size]
+        data = data[payload_size:]
+        msg_size = struct.unpack(">L", packed_msg_size)[0]
+        while len(data) < msg_size:
+            data += sock.recv(4096)
+        frame_data = data[:msg_size]
+        data = data[msg_size:]
+        xData = pickle.loads(frame_data, fix_imports=True, encoding="bytes")
+        return xData
     except:
         raise ConnectionError("Connection Error")
 
@@ -64,10 +64,12 @@ class Socket_Server_Host:
 
         PORT (:obj:`int`): Which port you'd like to host this server on.
 
-        on_connection_open (:obj:`def`): The function to call when you get a new connection.
+        on_connection_open (:obj:`def`): The function to call when you get a new connection. Gives Socket_Server_Client Class
 
         on_data_recv (:obj:`def`): The function to call when you receive data from a connection.
 
+        on_question (:obj:`def`): The function to call when you receive a question from a connection.
+
         on_connection_close (:obj:`def`, optional): The function to call when a connection is closed.
 
         daemon (:obj:`bool`, optional): If you'd like the server to close when the python file closes or is interrupted. 
@@ -135,6 +137,7 @@ class Socket_Server_Host:
                 connector = Socket_Server_Client(conn, addr, conID, self.on_data_recv, on_question=self.on_question, on_close=self.close_connection)
                 self.connections[conID] = connector
                 self.on_connection_open(connector)
+                time.sleep(0.05)
             except Exception as e:
                 self.running = False
                 raise e
@@ -177,37 +180,8 @@ class Socket_Server_Host:
 
 class Socket_Server_Client:
 
-    def __init__(self, socket, addr, conID, on_data, on_question, on_close):
-        """Host your own Socket server to allows connections to this computer.
-
-        Parameters
-        ----------
-        socket (:obj:`socket connection`): opened socket to remote connection
-
-        addr (:obj:`str`): address of the remote connection
-
-        conID (:obj:`def`): The connection ID of the remote connection
-
-        on_data (:obj:`def`): The function to call when you receive data from a connection.
-
-        on_close (:obj:`def`, optional): The function to call when a connection is closed.
-
-        Attributes
-        ----------
-
-        socket (:obj:`socket connection`): opened socket to remote connection
-
-        addr (:obj:`str`): address of the remote connection
-
-        conID (:obj:`def`): The connection ID of the remote connection
-
-        on_data (:obj:`def`): The function to call when you receive data from a connection.
-
-        on_close (:obj:`def`, optional): The function to call when a connection is closed.
-
-        running (:obj:`bool`): Is the server still running.
-
-        """
+    def __init__(self, socket, addr, conID, on_data, on_question, on_close, Heart_Beat=True, Heart_Beat_Wait=20):
+        """CLIENT for Socket_Server_Host"""
         self.socket = socket
         self.addr = addr
         self.conID = conID
@@ -219,6 +193,14 @@ class Socket_Server_Client:
         self.__ask_list__ = {}
         self.created = essentials.TimeStamp()
         threading.Thread(target=self.__data_rev__, daemon=True).start()
+        if Heart_Beat == True:
+            self.heart_beat_wait = Heart_Beat_Wait
+            threading.Thread(target=self.__heart_beat__, daemon=True).start()
+
+        def __heart_beat__(self):
+            while self.running:
+                self.send({"heart_beat_function": True})
+                time.sleep(self.heart_beat_wait)
 
     def shutdown(self):
         """
@@ -233,8 +215,14 @@ class Socket_Server_Client:
         except:
             pass
         self.running = False
-        self.socket.shutdown(socket.SHUT_RDWR)
-        self.socket.close()
+        try:
+            self.socket.shutdown(socket.SHUT_RDWR)
+        except:
+            pass
+        try:
+            self.socket.close()
+        except:
+            pass
         
 
     def send(self, data):
@@ -252,12 +240,15 @@ class Socket_Server_Client:
         except:
             self.shutdown()
 
-    def ask(self, data):
+    def ask(self, data, timeout=5):
         tok = essentials.CreateToken(20, self.__ask_list__)
         self.__ask_list__[tok] = False
         self.send({"function_ask_question": tok, "data": data})
         while self.__ask_list__[tok] == False:
-            pass
+            time.sleep(0.01)
+            timeout -= 0.01
+            if timeout <= 0:
+                raise TimeoutError("No response within time.")
         copyed = copy.deepcopy(self.__ask_list__[tok])
         del self.__ask_list__[tok]
         return copyed['data']
@@ -276,12 +267,15 @@ class Socket_Server_Client:
             except:
                 self.shutdown()
                 return
-            if type(data) == type({}) and 'function_ask_response' in data:
+            if type(data) == type({}) and 'heart_beat_function' in data:
+                pass
+            elif type(data) == type({}) and 'function_ask_response' in data:
                 self.__ask_list__[data['function_ask_response']] = data
             elif type(data) == type({}) and 'function_ask_question' in data:
                 self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
             else:
                 self.on_data(data, self)
+            time.sleep(0.05)
 
 class Socket_Question(object):
     def __init__(self, data, client, tok):
@@ -294,7 +288,7 @@ class Socket_Question(object):
 
 class Socket_Connector:
 
-    def __init__(self, HOST, PORT, on_data_recv, on_question, on_connection_close):
+    def __init__(self, HOST, PORT, on_data_recv, on_question, on_connection_close, Heart_Beat=True, Heart_Beat_Wait=10):
         """Host your own Socket server to allows connections to this computer.
 
         Parameters
@@ -328,13 +322,24 @@ class Socket_Connector:
         self.socket = ConnectorSocket(HOST, PORT)
         self.on_data_recv = on_data_recv
         threading.Thread(target=self.__data_rev__, daemon=True).start()
+        if Heart_Beat == True:
+            self.heart_beat_wait = Heart_Beat_Wait
+            threading.Thread(target=self.__heart_beat__, daemon=True).start()
 
-    def ask(self, data):
+    def __heart_beat__(self):
+        while self.running:
+            self.send({"heart_beat_function": True})
+            time.sleep(self.heart_beat_wait)
+
+    def ask(self, data, timeout=5):
         tok = essentials.CreateToken(20, self.__ask_list__)
         self.__ask_list__[tok] = False
         self.send({"function_ask_question": tok, "data": data})
         while self.__ask_list__[tok] == False:
-            pass
+            time.sleep(0.01)
+            timeout -= 0.01
+            if timeout <= 0:
+                raise TimeoutError("No response within time.")
         copyed = copy.deepcopy(self.__ask_list__[tok])
         del self.__ask_list__[tok]
         return copyed['data']
@@ -362,8 +367,14 @@ class Socket_Connector:
         """
         self.running = False
         self.on_connection_close(self)
-        self.socket.shutdown(socket.SHUT_RDWR)
-        self.socket.close()
+        try:
+            self.socket.shutdown(socket.SHUT_RDWR)
+        except:
+            pass
+        try:
+            self.socket.close()
+        except:
+            pass
     
     def __data_rev__(self):
         """
@@ -378,7 +389,9 @@ class Socket_Connector:
             except:
                 self.shutdown()
                 return
-            if type(data) == type({}) and 'function_ask_response' in data:
+            if type(data) == type({}) and 'heart_beat_function' in data:
+                pass
+            elif type(data) == type({}) and 'function_ask_response' in data:
                 self.__ask_list__[data['function_ask_response']] = data
             elif type(data) == type({}) and 'function_ask_question' in data:
                 self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
